@@ -23,9 +23,8 @@ GenerateGo()
 GenerateJava()
 GenerateKotlin()
 GenerateLobster()
-# TODO these doesn't respect the output prefix, fix and reenable
-# GenerateLua()
-# GenerateNim()
+GenerateLua()
+GenerateNim()
 GeneratePhp()
 GeneratePython()
 GenerateRust()
